-module(ircbot).
-author("kevin@scaldeferri.com").
% based on http://blog.jroes.net/2007/08/simple-erlang-irc-bot.html
% by jonathan.roes@gmail.com

-export([start/0, loop/2]).

-include_lib("xmerl/include/xmerl.hrl").

-define(host, "irc.perl.org").
-define(port, 6667).
-define(feed, "http://search.cpan.org/uploads.rdf").
-define(nickname, "erlbot").
-define(channel, "#pdxtest").

start () ->
    connect(?host, ?port).

% Connect to an IRC server with a given Host and Port.  Set up the TCP option to
% give us messages on a line-by-line basis.
connect(Host, Port) ->
    application:start(inets),
    {ok, Sock} = gen_tcp:connect(Host, Port, [{packet, line}]),
    % According to RFC1459, we need to tell the server our nickname and username
    gen_tcp:send(Sock, "NICK " ++ ?nickname ++ "\r\n"),
    gen_tcp:send(Sock, "USER PDX.pm erlang bot blah blah\r\n"),
    loop(Sock, sets:new()).

% Now that we're connected, receive TCP messages and parse them.
loop(Sock, Seen) ->
    receive
        {tcp, Sock, Data} ->
            io:format("[~w] Received: ~s", [Sock, Data]),
            parse_line(Sock, string:tokens(Data, ": ,\r\n")),
            loop(Sock, Seen);
        check_feed ->
            NewSeen = check_feed(Sock, Seen),
            timer:send_after (timer:minutes(1), check_feed),
            loop(Sock, NewSeen);
        code_upgrade ->
            ?MODULE:loop(Sock, Seen);
        quit ->
            io:format("[~w] Received quit message, exiting...~n", [Sock]),
            gen_tcp:close(Sock),
            exit(stopped)
    end.


% If the second token is "376", then join our channel.  376 indicates End of MOTD.
parse_line(Sock, [_,"376"|_]) ->
    gen_tcp:send(Sock, "JOIN :" ++ ?channel ++ "\r\n"),
    timer:send_after (timer:seconds(5), check_feed);

% The server will periodically send PINGs and expect you to PONG back to make sure
% you haven't lost the connection.
parse_line(Sock, ["PING"|Rest]) ->
    gen_tcp:send(Sock, "PONG " ++ Rest ++ "\r\n");

parse_line(_, _) ->
    0.

% This just helps us write a PRIVMSG back to a client without having to type
% the newlines and :'s ourselves so much.  It'll be more useful later.
irc_privmsg(Sock, To, Message) ->
    io:format("Sending: ~s~n", ["PRIVMSG " ++ To ++ " :" ++ Message]),
    gen_tcp:send(Sock, "PRIVMSG " ++ To ++ " :" ++ Message ++ "\r\n").

check_feed(Sock,Seen) ->
    case http:request(?feed) of
        { ok, {_Status, _Headers, Body }} ->
            { Xml, _Rest } = xmerl_scan:string(Body),
            Entries = get_entries(xmerl_xpath:string("//item",Xml)),
            process_entries(Entries, Seen, Sock);
        _ ->
            Seen
    end.

get_entries([]) -> [];
get_entries([Node|Rest]) ->
    Title  = reconstitute_text("title", Node),
    Link   = reconstitute_text("link", Node),
    Author = reconstitute_text("dc:creator", Node),
    Entry  = Title ++ ", by " ++ Author ++ ", " ++ Link,
    [Entry | get_entries(Rest)].

% The existence of a entity (e.g. &amp;) in the title causes multiple
% text nodes to be created.  We'll just concatenate the list.
reconstitute_text(ElementName, Node) ->
    lists:concat([Piece || #xmlText{value=Piece} <- xmerl_xpath:string(ElementName ++ "/text()", Node)]).

process_entries([], Seen, _Sock) ->
    Seen;
process_entries([Entry|Rest], Seen, Sock) ->
    case sets:is_element(Entry, Seen) of
        true ->
            process_entries(Rest, Seen, Sock);
        false ->
            io:fwrite ("New!!! ~s~n", [Entry]),
            irc_privmsg(Sock, ?channel, "New Post: " ++ Entry),
            % try to avoid getting kicked for flooding
            receive
                after timer:seconds(1) -> true
            end,
            NewSeen = sets:add_element(Entry, Seen),
            process_entries(Rest, NewSeen, Sock)
    end.
