#!/usr/bin/env perl

# This is a simple IRC bot based on http://poe.perl.org/?POE_Cookbook/IRC_Bots
# It echoes RSS updates to search.cpan.org into the IRC channel

use warnings;
use strict;

#sub POE::Kernel::ASSERT_DEFAULT () { 1 };
use POE;
use POE::Component::IRC;
use POE::Component::Client::HTTP;
use HTTP::Request;
use XML::XPath;

use constant {
  CHANNEL => "#pdxtest",
  HOST    => "irc.perl.org",
  PORT    => 6667,
  NICK    => "poebot",
  FEED    => "http://search.cpan.org/uploads.rdf"
};

# Create the component that will represent an IRC network.
my $irc = POE::Component::IRC->spawn();

POE::Component::Client::HTTP->spawn(
  Alias       => 'http_client',
  Timeout     => 30,
);

# Create the bot session.  The new() call specifies the events the bot
# knows about and the functions that will handle those events.
POE::Session->create(
    inline_states => {
        _start     => \&bot_start,
        irc_001    => \&on_connect,
        check_feed => \&check_feed,
        process_feed => \&process_feed,
    },
);

# The bot session has started.  Register this bot with the "magnet"
# IRC component.  Select a nickname.  Connect to a server.
sub bot_start {
  my ($kernel, $heap)  = @_[KERNEL, HEAP];

  $heap->{seen} = {};

  $irc->yield( register => qw(001) );

  $irc->yield( connect =>
               { Nick     => NICK,
                 Username => 'cookbot',
                 Ircname  => 'PDX.pm POE bot',
                 Server   => HOST,
                 Port     => PORT,
                 Flood    => 0, # This is the default.  Here for demontration purposes only.
               }
             );
}

# The bot has successfully connected to a server.  Join a channel.
sub on_connect {
  $irc->yield( join => CHANNEL );
  $_[KERNEL]->delay("check_feed", 5);
}

sub check_feed {
  $_[KERNEL]->post('http_client',
                   'request',
                   'process_feed',
                   HTTP::Request->new(GET => FEED));
}

sub process_feed {
  my ( $kernel, $heap, $res ) = @_[ KERNEL, HEAP, ARG1 ];

  my $response = $res->[0];
  if ($response->is_success) {
    my $xp = XML::XPath->new(xml => $response->content);
    for my $entry (map {get_entry($xp, $_)} $xp->findnodes("//item")) {
      if (! $heap->{seen}{$entry}) {
        print("New!!! $entry\n");
        $irc->yield( privmsg => CHANNEL, "New Post: $entry");
        $heap->{seen}{$entry} = 1;
      }
    }
  }

  $kernel->delay("check_feed", 60);
}

sub get_entry {
  my ($xp, $node) = @_;

  my $title = get_text($xp, "title", $node);
  my $link = get_text($xp, "link", $node);
  my $author = get_text($xp, "dc:creator", $node);
  return "$title, by $author, $link";
}

sub get_text {
  my ($xp, $element_name, $node) = @_;
  return
    join("", map {$_->getValue()} $xp->findnodes("${element_name}/text()", $node));
}

# Run the bot until it is done.
$poe_kernel->run();
exit 0;

